@@ -3,10 +3,11 @@
 
   rbgobj_object.c -
 
-  $Author: sakai $
-  $Date: 2003/11/06 16:53:43 $
+  $Author: mutoh $
+  $Date: 2005/11/03 11:53:16 $
 
-  Copyright (C) 2002,2003  Masahiro Sakai
+  Copyright (C) 2002-2004  Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2003  Masahiro Sakai
 
   This file is derived from rbgtkobject.c in Ruby/Gtk distribution.
   rbgtkobject.c -
@@ -26,7 +27,7 @@ void
 rbgobj_add_abstract_but_create_instance_class(gtype)
     GType gtype;
 {
-    RGObjClassInfo* cinfo = (RGObjClassInfo*)rbgobj_lookup_class_by_gtype(gtype);
+    RGObjClassInfo* cinfo = (RGObjClassInfo*)GTYPE2CINFO(gtype);
     cinfo->flags |= RBGOBJ_ABSTRACT_BUT_CREATABLE;
 }
 
@@ -64,8 +65,11 @@ gobj_s_gobject_new(argc, argv, self)
 
     rb_scan_args(argc, argv, "01", &params_hash);
 
+    if (!NIL_P(params_hash))
+        Check_Type(params_hash, T_HASH);
+
     if (cinfo->klass != self)
-        rb_raise(rb_eTypeError, "%s isn't registerd class",
+        rb_raise(rb_eTypeError, "%s isn't registered class",
                  rb_class2name(self));
 
     gobj = rbgobj_gobject_new(cinfo->gtype, params_hash);
@@ -92,7 +96,7 @@ gobj_s_install_property(int argc, VALUE* argv, VALUE self)
     VALUE pspec_obj, prop_id;
 
     if (cinfo->klass != self)
-        rb_raise(rb_eTypeError, "%s isn't registerd class",
+        rb_raise(rb_eTypeError, "%s isn't registered class",
                  rb_class2name(self));
 
     rb_scan_args(argc, argv, "11", &pspec_obj, &prop_id);
@@ -142,7 +146,7 @@ static VALUE
 gobj_s_properties(int argc, VALUE* argv, VALUE self)
 {
     GObjectClass* oclass = g_type_class_ref(CLASS2GTYPE(self));
-    gint n_properties;
+    guint n_properties;
     GParamSpec** props;
     VALUE inherited_too;
     VALUE ary;
@@ -159,7 +163,7 @@ gobj_s_properties(int argc, VALUE* argv, VALUE self)
             || GTYPE2CLASS(props[i]->owner_type) == self)
             rb_ary_push(ary, rb_str_new2(props[i]->name));
     }
-
+    g_free(props);
     g_type_class_unref(oclass);
     return ary;
 }
@@ -236,6 +240,7 @@ gobj_set_property(self, prop_name, val)
         // FIXME: use rb_ensure to call g_value_unset()
         RValueToGValueFunc setter = NULL;
         GValue gval = {0,};
+
         g_value_init(&gval, G_PARAM_SPEC_VALUE_TYPE(pspec));
 
         {
@@ -247,15 +252,17 @@ gobj_set_property(self, prop_name, val)
                     Data_Get_Struct(obj, void, setter);
             }
         }
-
-        if (setter)
+        if (setter) {
             setter(val, &gval);
-        else {
+        } else {
             rbgobj_rvalue_to_gvalue(val, &gval);
         }
 
         g_object_set_property(RVAL2GOBJ(self), name, &gval);
         g_value_unset(&gval);
+
+        G_CHILD_SET(self, rb_intern(name), val);
+
         return self;
     }
 }
@@ -297,8 +304,12 @@ gobj_get_property(self, prop_name)
 
         g_value_init(&gval, G_PARAM_SPEC_VALUE_TYPE(pspec));
         g_object_get_property(RVAL2GOBJ(self), name, &gval);
+
         ret = getter ? getter(&gval) : GVAL2RVAL(&gval);
         g_value_unset(&gval);
+
+        G_CHILD_SET(self, rb_intern(name), ret);
+
         return ret;
     }
 }
@@ -380,6 +391,9 @@ gobj_initialize(argc, argv, self)
     
     rb_scan_args(argc, argv, "01", &params_hash);
 
+    if (!NIL_P(params_hash))
+        Check_Type(params_hash, T_HASH);
+
     gobj = rbgobj_gobject_new(RVAL2GTYPE(self), params_hash);
 
     if (is_gtkobject(gobj)){
@@ -564,7 +578,7 @@ type_register(int argc, VALUE* argv, VALUE self)
 
         {
             GType parent = g_type_parent(type);
-            const RGObjClassInfo* cinfo = rbgobj_lookup_class_by_gtype(parent);
+            const RGObjClassInfo* cinfo =  GTYPE2CINFO(parent);
             VALUE m = rb_define_module_under(self, RubyGObjectHookModule);
 
             if (! (cinfo->flags & RBGOBJ_DEFINED_BY_RUBY)) {
@@ -598,7 +612,7 @@ Init_gobject_gobject()
 #ifndef HAVE_RB_DEFINE_ALLOC_FUNC
     rb_define_singleton_method(cGObject, "allocate", &gobj_s_allocate, 0);
 #else
-    rb_define_alloc_func(cGObject, gobj_s_allocate);
+    rb_define_alloc_func(cGObject, (VALUE(*)_((VALUE)))gobj_s_allocate);
 #endif
     rb_define_singleton_method(cGObject, "new!", gobj_s_gobject_new, -1);
 
