@@ -1,23 +1,23 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/* $Id: rbgnome-canvas-item.c,v 1.14 2004/03/05 15:56:29 mutoh Exp $ */
+/* $Id: rbgnome-canvas-item.c,v 1.21 2005/10/01 19:44:15 mutoh Exp $ */
 
 /* Gnome::CanvasItem widget for Ruby/Gnome
- * Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
+ * Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
  * Copyright (C) 2001 Neil Conway <neilconway@rogers.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "rbgnomecanvas.h"
@@ -57,14 +57,21 @@ citem_intialize(self, parent, hash)
     GnomeCanvasItem *item;
     GnomeCanvasGroup *group;
 
+    if (! rb_obj_is_kind_of(parent, GTYPE2CLASS(GNOME_TYPE_CANVAS_GROUP))){
+        rb_raise(rb_eArgError, "the 1st parameter should be Gnome::CanvasGroup");
+    }
+
     group = GNOME_CANVAS_GROUP(RVAL2GOBJ(parent));
     item = GNOME_CANVAS_ITEM(g_object_new(RVAL2GTYPE(self), NULL));
     RBGTK_INITIALIZE(self, item);
+    g_object_ref(group);
+    g_object_freeze_notify(G_OBJECT(item));
 
-    item->parent = GNOME_CANVAS_ITEM(group);
-    item->canvas = item->parent->canvas;
-    rbgutil_set_properties(self, hash);
     citem_do_construct(item, group, NULL);
+    rbgutil_set_properties(self, hash);
+
+    g_object_thaw_notify(G_OBJECT(item));
+
     return Qnil;
 }
 
@@ -92,7 +99,7 @@ citem_affine_relative(self, affine)
     VALUE self, affine;
 {
     gnome_canvas_item_affine_relative(_SELF(self),
-                                      rbart_get_art_affine(affine));
+                                      get_art_affine(affine));
     return self;
 }
 
@@ -101,7 +108,7 @@ citem_affine_absolute(self, affine)
     VALUE self, affine;
 {
     gnome_canvas_item_affine_absolute(_SELF(self),
-                                      rbart_get_art_affine(affine));
+                                      get_art_affine(affine));
     return self;
 }
 
@@ -199,7 +206,7 @@ citem_i2c_affine(self)
 {
     double affine[6];
     gnome_canvas_item_i2c_affine(_SELF(self), affine);
-    return rbart_make_art_affine(affine);
+    return make_art_affine(affine);
 }
 
 static VALUE
@@ -208,7 +215,7 @@ citem_i2w_affine(self)
 {
     double affine[6];
     gnome_canvas_item_i2w_affine(_SELF(self), affine);
-    return rbart_make_art_affine(affine);
+    return make_art_affine(affine);
 }
 
 static VALUE
@@ -244,9 +251,13 @@ citem_get_bounds(self)
                        rb_float_new(x2), rb_float_new(y2));
 }
 
-#if 0 /* This should be used only by item implementations. */
-void gnome_canvas_item_request_update (GnomeCanvasItem *item);
-#endif
+static VALUE
+citem_request_update(self)
+    VALUE self;
+{
+    gnome_canvas_item_request_update(_SELF(self));
+    return self;
+}
 
 static VALUE
 citem_parent(self)
@@ -267,7 +278,7 @@ citem_shape_set_path_def(self, def)
     VALUE self, def;
 {
     gnome_canvas_shape_set_path_def(GNOME_CANVAS_SHAPE(RVAL2GOBJ(self)), 
-                                    (GnomeCanvasPathDef*)RVAL2BOXED(def, rbgno_canvas_path_def_get_type()));
+                                    (GnomeCanvasPathDef*)RVAL2BOXED(def, GNOME_TYPE_CANVAS_PATH_DEF));
     return self;
 }
 
@@ -275,7 +286,7 @@ static VALUE
 citem_shape_get_path_def(self)
     VALUE self;
 {
-    return BOXED2RVAL(gnome_canvas_shape_get_path_def(GNOME_CANVAS_SHAPE(RVAL2GOBJ(self))), rbgno_canvas_path_def_get_type());
+    return BOXED2RVAL(gnome_canvas_shape_get_path_def(GNOME_CANVAS_SHAPE(RVAL2GOBJ(self))), GNOME_TYPE_CANVAS_PATH_DEF);
 }
 
 void
@@ -305,6 +316,7 @@ Init_gnome_canvas_item(mGnome)
     rb_define_method(gnoCanvasItem, "reparent", citem_reparent, 1);
     rb_define_method(gnoCanvasItem, "grab_focus", citem_grab_focus, 0);
     rb_define_method(gnoCanvasItem, "bounds", citem_get_bounds, 0);
+    rb_define_method(gnoCanvasItem, "request_update", citem_request_update, 0);
     rb_define_method(gnoCanvasItem, "parent", citem_parent, 0);
     rb_define_method(gnoCanvasItem, "canvas", citem_canvas, 0);
 
