@@ -3,11 +3,10 @@
 
   rbgdkdraw.c -
 
-  $Author: mutoh $
-  $Date: 2004/03/05 16:24:30 $
+  $Author: ggc $
+  $Date: 2005/09/30 20:17:59 $
 
-  Copyright (C) 2002-2004 Masao Mutoh
-  Copyright (C) 2002,2003 Masao Mutoh
+  Copyright (C) 2002-2005 Masao Mutoh
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -18,6 +17,9 @@
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #endif
+#ifdef HAVE_RB_CAIRO_H
+#include <rb_cairo.h>
+#endif
 
 #define _SELF(s) GDK_DRAWABLE(RVAL2GOBJ(s))
 
@@ -229,6 +231,39 @@ gdkdraw_draw_poly(self, gc, filled, pnts)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,6,0)
+/*
+  trapezoids = [[y1, x11, x21, y2, x12, x22], ...]
+ */
+static VALUE
+gdkdraw_draw_trapezoids(self, gc, trapezoids)
+    VALUE self, gc, trapezoids;
+{
+    GdkTrapezoid *gtrapezoids;
+    gint i, len;
+
+    Check_Type(trapezoids, T_ARRAY);
+
+    len = RARRAY(trapezoids)->len;
+
+    gtrapezoids = ALLOCA_N(GdkTrapezoid, len);
+    for (i = 0; i < len; i++) {
+        Check_Type(RARRAY(trapezoids)->ptr[i], T_ARRAY);
+        if (RARRAY(RARRAY(trapezoids)->ptr[i])->len < 6) {
+            rb_raise(rb_eArgError, "trapezoids %d should be array of size 6", i);
+        }
+        gtrapezoids[i].y1 = NUM2DBL(RARRAY(RARRAY(trapezoids)->ptr[i])->ptr[0]);
+        gtrapezoids[i].x11 = NUM2DBL(RARRAY(RARRAY(trapezoids)->ptr[i])->ptr[1]);
+        gtrapezoids[i].x21 = NUM2DBL(RARRAY(RARRAY(trapezoids)->ptr[i])->ptr[2]);
+        gtrapezoids[i].y2 = NUM2DBL(RARRAY(RARRAY(trapezoids)->ptr[i])->ptr[3]);
+        gtrapezoids[i].x12 = NUM2DBL(RARRAY(RARRAY(trapezoids)->ptr[i])->ptr[4]);
+        gtrapezoids[i].x22 = NUM2DBL(RARRAY(RARRAY(trapezoids)->ptr[i])->ptr[5]);
+    }
+    gdk_draw_trapezoids(_SELF(self), GDK_GC(RVAL2GOBJ(gc)), gtrapezoids, len);
+    return self; 
+}
+#endif
+
 static VALUE
 gdkdraw_glyphs(self, gc, font, x, y, glyphs)
     VALUE self, gc, font, x, y, glyphs;
@@ -239,6 +274,20 @@ gdkdraw_glyphs(self, gc, font, x, y, glyphs)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,6,0)
+static VALUE
+gdkdraw_glyphs_transformed(self, gc, matrix, font, x, y, glyphs)
+    VALUE self, gc, matrix, font, x, y, glyphs;
+{
+    gdk_draw_glyphs_transformed(_SELF(self), GDK_GC(RVAL2GOBJ(gc)), 
+                                NIL_P(matrix) ? (PangoMatrix*)NULL : (PangoMatrix*)(RVAL2BOXED(matrix, PANGO_TYPE_MATRIX)),
+                                PANGO_FONT(RVAL2GOBJ(font)),
+                                NUM2INT(x), NUM2INT(y), 
+                                (PangoGlyphString*)(RVAL2BOXED(glyphs, PANGO_TYPE_GLYPH_STRING)));
+    return self;
+}
+#endif
+
 static VALUE
 gdkdraw_layout_line(argc, argv, self)
     int argc;
@@ -249,19 +298,11 @@ gdkdraw_layout_line(argc, argv, self)
 
     rb_scan_args(argc, argv, "42", &gc, &x, &y, &line, &fg, &bg);
 
-    if (argc == 4){
-        gdk_draw_layout_line(_SELF(self), GDK_GC(RVAL2GOBJ(gc)), 
-                             NUM2INT(x), NUM2INT(y), 
-                             (PangoLayoutLine*)RVAL2BOXED(line, PANGO_TYPE_LAYOUT_LINE));
-    } else if (argc == 6){
-        gdk_draw_layout_line_with_colors(_SELF(self), GDK_GC(RVAL2GOBJ(gc)), 
-                                         NUM2INT(x), NUM2INT(y), 
-                                         (PangoLayoutLine*)RVAL2BOXED(line, PANGO_TYPE_LAYOUT_LINE),
-                                         (GdkColor*)RVAL2BOXED(fg, GDK_TYPE_COLOR),
-                                         (GdkColor*)RVAL2BOXED(bg, GDK_TYPE_COLOR));
-    } else {
-        rb_raise(rb_eArgError, "Wrong arguments");
-    }
+    gdk_draw_layout_line_with_colors(_SELF(self), GDK_GC(RVAL2GOBJ(gc)),
+                                     NUM2INT(x), NUM2INT(y),
+                                     (PangoLayoutLine*)RVAL2BOXED(line, PANGO_TYPE_LAYOUT_LINE),
+                                     NIL_P(fg) ? NULL : (GdkColor*)RVAL2BOXED(fg, GDK_TYPE_COLOR),
+                                     NIL_P(bg) ? NULL : (GdkColor*)RVAL2BOXED(bg, GDK_TYPE_COLOR));
 
     return self;
 }
@@ -276,17 +317,10 @@ gdkdraw_layout(argc, argv, self)
 
     rb_scan_args(argc, argv, "42", &gc, &x, &y, &layout, &fg, &bg);
 
-    if (argc == 4){
-        gdk_draw_layout(_SELF(self), GDK_GC(RVAL2GOBJ(gc)), 
-                        NUM2INT(x), NUM2INT(y), PANGO_LAYOUT(RVAL2GOBJ(layout)));
-    } else if (argc == 6){
-        gdk_draw_layout_with_colors( _SELF(self), GDK_GC(RVAL2GOBJ(gc)), 
-                                     NUM2INT(x), NUM2INT(y), PANGO_LAYOUT(RVAL2GOBJ(layout)),
-                                     (GdkColor*)RVAL2BOXED(fg, GDK_TYPE_COLOR),
-                                     (GdkColor*)RVAL2BOXED(bg, GDK_TYPE_COLOR));
-    } else {
-        rb_raise(rb_eArgError, "Wrong arguments");
-    }
+    gdk_draw_layout_with_colors(_SELF(self), GDK_GC(RVAL2GOBJ(gc)),
+                                NUM2INT(x), NUM2INT(y), PANGO_LAYOUT(RVAL2GOBJ(layout)),
+                                NIL_P(fg) ? NULL : (GdkColor*)RVAL2BOXED(fg, GDK_TYPE_COLOR),
+                                NIL_P(bg) ? NULL : (GdkColor*)RVAL2BOXED(bg, GDK_TYPE_COLOR));
 
     return self;
 }
@@ -323,6 +357,19 @@ gdkdraw_get_image(self, x, y, w, h)
                                             NUM2INT(w), NUM2INT(h)));
 }
 
+#if GTK_CHECK_VERSION(2,4,0)
+static VALUE
+gdkdraw_copy_to_image(self, image, xsrc, ysrc, xdst, ydst, w, h)
+    VALUE self, image, xsrc, ysrc, xdst, ydst, w, h;
+{
+    return GOBJ2RVAL(gdk_drawable_copy_to_image(_SELF(self), 
+                                                GDK_IMAGE(RVAL2GOBJ(image)),
+                                                NUM2INT(xsrc), NUM2INT(ysrc),
+                                                NUM2INT(xdst), NUM2INT(ydst),
+                                                NUM2INT(w), NUM2INT(h)));
+}
+#endif
+
 #ifdef GDK_WINDOWING_X11
 static VALUE
 gdkdraw_get_xid(self)
@@ -348,6 +395,16 @@ gdkdraw_get_screen(self)
 }
 #endif
 
+#if GTK_CHECK_VERSION(2,8,0)
+#  ifdef HAVE_RB_CAIRO_H
+static VALUE
+gdkdraw_cairo_create(self)
+    VALUE self;
+{
+    return CRCONTEXT2RVAL(gdk_cairo_create(_SELF(self)));
+}
+#  endif
+#endif
 
 void
 Init_gtk_gdk_draw()
@@ -372,12 +429,22 @@ Init_gtk_gdk_draw()
     rb_define_method(gdkDrawable, "draw_rectangle", gdkdraw_draw_rect, 6);
     rb_define_method(gdkDrawable, "draw_arc", gdkdraw_draw_arc, 8);
     rb_define_method(gdkDrawable, "draw_polygon", gdkdraw_draw_poly, 3);
+#if GTK_CHECK_VERSION(2,6,0)
+    rb_define_method(gdkDrawable, "draw_trapezoids", gdkdraw_draw_trapezoids, 2);
+#endif
     rb_define_method(gdkDrawable, "draw_glyphs", gdkdraw_glyphs, 5);
+#if GTK_CHECK_VERSION(2,6,0)
+    rb_define_method(gdkDrawable, "draw_glyphs_transformed", gdkdraw_glyphs_transformed, 6);
+#endif
     rb_define_method(gdkDrawable, "draw_layout_line", gdkdraw_layout_line, -1);
     rb_define_method(gdkDrawable, "draw_layout", gdkdraw_layout, -1);
     rb_define_method(gdkDrawable, "draw_drawable", gdkdraw_draw_drawable, 8);
     rb_define_method(gdkDrawable, "draw_image", gdkdraw_draw_image, 8);
     rb_define_method(gdkDrawable, "get_image", gdkdraw_get_image, 4);
+#if GTK_CHECK_VERSION(2,4,0)
+    rb_define_method(gdkDrawable, "copy_to_image", gdkdraw_copy_to_image, 7);
+#endif
+
 #ifdef GDK_WINDOWING_X11
     rb_define_method(gdkDrawable, "xid", gdkdraw_get_xid, 0);
 #endif
@@ -386,6 +453,13 @@ Init_gtk_gdk_draw()
     rb_define_method(gdkDrawable, "screen", gdkdraw_get_screen, 0);
 #endif
     
+#if GTK_CHECK_VERSION(2,8,0)
+#  ifdef HAVE_RB_CAIRO_H
+    rb_define_method(gdkDrawable, "create_cairo_context",
+                     gdkdraw_cairo_create, 0);
+#  endif
+#endif
+    
 #ifdef GDK_WINDOWING_X11
     G_DEF_CLASS3("GdkDrawableImplX11", "DrawableImplX11", mGdk);
 #elif defined(GDK_WINDOWING_WIN32)
