@@ -4,9 +4,9 @@
   rbgtkcontainer.c -
 
   $Author: mutoh $
-  $Date: 2003/06/26 15:15:32 $
+  $Date: 2005/11/06 04:44:24 $
 
-  Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -28,32 +28,14 @@ static VALUE
 cont_is_resize_container(self)
     VALUE self;
 {
-    return GTK_IS_RESIZE_CONTAINER(_SELF(self)) ? Qtrue : Qfalse;
-}
-
-static VALUE
-cont_add(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
-{
-    VALUE other, properties;
-    GtkWidget *child;
-
-    rb_scan_args(argc, argv, "11", &other, &properties);
-
-    child = GTK_WIDGET(RVAL2GOBJ(other));
-    gtk_container_add(_SELF(self), child);
-    if (! NIL_P(properties)){
-        G_SET_PROPERTIES(other, properties);
-    }
-    return self;
+    return CBOOL2RVAL(GTK_IS_RESIZE_CONTAINER(_SELF(self)));
 }
 
 static VALUE
 cont_remove(self, other)
     VALUE self, other;
 {
+    G_CHILD_REMOVE(self, other);
     gtk_container_remove(_SELF(self), GTK_WIDGET(RVAL2GOBJ(other)));
     return self;
 }
@@ -82,6 +64,22 @@ cont_foreach(argc, argv, self)
     return self;
 }
 
+static VALUE
+cont_forall(argc, argv, self)
+    int argc;
+    VALUE *argv;
+    VALUE self;
+{
+    VALUE callback;
+
+    rb_scan_args(argc, argv, "01", &callback);
+    if (NIL_P(callback)) {
+        callback = G_BLOCK_PROC();
+    }
+    gtk_container_forall(_SELF(self), exec_callback, (gpointer)callback);
+    return self;
+}
+
 static VALUE
 cont_get_children(self)
     VALUE self;
@@ -111,7 +109,7 @@ cont_set_focus_vadjustment(self, adjustment)
     VALUE self, adjustment;
 {
     gtk_container_set_focus_vadjustment(_SELF(self),
-                                        GTK_ADJUSTMENT(RVAL2GOBJ(adjustment)));
+                                        NIL_P(adjustment) ? NULL : GTK_ADJUSTMENT(RVAL2GOBJ(adjustment)));
     return self;
 }
 
@@ -120,7 +118,7 @@ cont_set_focus_hadjustment(self, adjustment)
     VALUE self, adjustment;
 {
     gtk_container_set_focus_hadjustment(_SELF(self),
-                                        GTK_ADJUSTMENT(RVAL2GOBJ(adjustment)));
+                                        NIL_P(adjustment) ? NULL : GTK_ADJUSTMENT(RVAL2GOBJ(adjustment)));
     return self;
 }
 
@@ -128,14 +126,16 @@ static VALUE
 cont_get_focus_vadjustment(self)
     VALUE self;
 {
-    return GOBJ2RVAL(gtk_container_get_focus_vadjustment(_SELF(self)));
+    GtkAdjustment* adj = gtk_container_get_focus_vadjustment(_SELF(self));
+    return adj ? GOBJ2RVAL(adj) : Qnil;
 }
 
 static VALUE
 cont_get_focus_hadjustment(self)
     VALUE self;
 {
-    return GOBJ2RVAL(gtk_container_get_focus_hadjustment(_SELF(self)));
+    GtkAdjustment* adj = gtk_container_get_focus_hadjustment(_SELF(self));
+    return adj ? GOBJ2RVAL(adj) : Qnil;
 }
 
 static VALUE
@@ -248,6 +248,9 @@ cont_child_get_property(self, child, prop_name)
                                          GTK_WIDGET(RVAL2GOBJ(child)),
                                          name , &gval);
         ret = getter ? getter(&gval) : GVAL2RVAL(&gval);
+
+        G_CHILD_ADD(child, ret);
+
         g_value_unset(&gval);
         return ret;
     }
@@ -294,6 +297,8 @@ cont_child_set_property(self, child, prop_name, val)
                 rbgobj_rvalue_to_gvalue(val, &gval);
         }
 
+        G_CHILD_ADD(child, val);
+
         gtk_container_child_set_property(GTK_CONTAINER(RVAL2GOBJ(self)), 
                                          GTK_WIDGET(RVAL2GOBJ(child)), name, &gval);
 
@@ -301,6 +306,43 @@ cont_child_set_property(self, child, prop_name, val)
         return self;
     }
 }
+
+static VALUE
+cont_add(argc, argv, self)
+    int argc;
+    VALUE *argv;
+    VALUE self;
+{
+    VALUE other, properties;
+    GtkWidget *child;
+
+    rb_scan_args(argc, argv, "11", &other, &properties);
+
+    child = GTK_WIDGET(RVAL2GOBJ(other));
+    gtk_widget_freeze_child_notify(child);
+    gtk_container_add(_SELF(self), child);
+
+    G_CHILD_ADD(self, other);
+
+    if (child->parent && (! NIL_P(properties))){
+        int i;
+        VALUE ary;
+        GObject* obj;
+                                                                                
+        Check_Type(properties, T_HASH);
+        ary = rb_funcall(properties, rb_intern("to_a"), 0);
+        obj = RVAL2GOBJ(self);
+        
+        for (i = 0; i < RARRAY(ary)->len; i++) {
+            cont_child_set_property(self, other, 
+                       RARRAY(RARRAY(ary)->ptr[i])->ptr[0],
+                       RARRAY(RARRAY(ary)->ptr[i])->ptr[1]);
+        }
+    }
+    gtk_widget_thaw_child_notify(child);
+    return self;
+}
+
 /*
 void        gtk_container_child_get_valist  (GtkContainer *container,
                                              GtkWidget *child,
@@ -311,11 +353,6 @@ void        gtk_container_child_set_valist  (GtkContainer *container,
                                              const gchar *first_property_name,
                                              va_list var_args);
 */
-/* Use gtk_container_foreach instead
-void        gtk_container_forall            (GtkContainer *container,
-                                             GtkCallback callback,
-                                             gpointer callback_data);
-*/
 
 static VALUE
 cont_propagate_expose(self, child, event)
@@ -397,7 +434,7 @@ cont_s_child_property(self, property_name)
     g_type_class_unref(oclass);
     return result;
 }
-/*
+
 static VALUE
 cont_s_install_child_property(argc, argv, self)
     int argc;
@@ -406,24 +443,25 @@ cont_s_install_child_property(argc, argv, self)
 {
     const RGObjClassInfo* cinfo = rbgobj_lookup_class(self);
     GtkContainerClass* gclass;
+    VALUE spec, prop_id;
     GParamSpec* pspec;
-    VALUE pspec_obj, prop_id;
+
+    rb_scan_args(argc, argv, "11", &spec, &prop_id);
+
+    pspec = G_PARAM_SPEC(RVAL2GOBJ(spec));
 
     if (cinfo->klass != self)
         rb_raise(rb_eTypeError, "%s isn't registerd class",
                  rb_class2name(self));
 
-    rb_scan_args(argc, argv, "11", &pspec_obj, &prop_id);
-    pspec = G_PARAM_SPEC(RVAL2GOBJ(pspec_obj));
+    gclass = GTK_CONTAINER_CLASS(g_type_class_ref(cinfo->gtype));
+    gtk_container_class_install_child_property(gclass, 
+                                               NIL_P(prop_id) ? 1 : NUM2UINT(prop_id),
+                                               pspec);
 
-    gclass = (GtkContainerClass *)g_type_class_ref(cinfo->gtype);
-    gtk_container_class_install_child_property(
-        gclass, NIL_P(prop_id) ? 1: NUM2INT(prop_id), 
-        pspec);
-
-    return Qnil;
+    return self;
 }
-*/
+
 static VALUE
 cont_s_child_properties(argc, argv, self)
     int argc;
@@ -431,13 +469,14 @@ cont_s_child_properties(argc, argv, self)
     VALUE self;
 {
     GObjectClass* oclass = g_type_class_ref(CLASS2GTYPE(self));
-    gint n_properties;
+    guint n_properties;
     GParamSpec** props;
     VALUE inherited_too;
     VALUE ary;
     int i;
 
-    rb_scan_args(argc, argv, "01", &inherited_too);
+    if (rb_scan_args(argc, argv, "01", &inherited_too) == 0)
+        inherited_too = Qtrue;
 
     props = gtk_container_class_list_child_properties(oclass, &n_properties);
 
@@ -445,13 +484,243 @@ cont_s_child_properties(argc, argv, self)
     for (i = 0; i < n_properties; i++){
         if (RTEST(inherited_too)
             || GTYPE2CLASS(props[i]->owner_type) == self)
-            rb_ary_push(ary, rb_str_new2(props[i]->name));
+            rb_ary_push(ary, CSTR2RVAL(props[i]->name));
     }
-
+    g_free(props);
     g_type_class_unref(oclass);
     return ary;
 }
 
+/**********************************************************************/
+
+static VALUE proc_mod_eval;
+static GQuark q_ruby_setter;
+static GQuark q_ruby_getter;
+
+static VALUE
+cont_initialize(argc, argv, self)
+    int argc;
+    VALUE *argv;
+    VALUE self;
+{
+    VALUE params_hash;
+    GObject* gobj;
+    
+    rb_scan_args(argc, argv, "01", &params_hash);
+
+    if (!NIL_P(params_hash))
+        Check_Type(params_hash, T_HASH);
+
+    gobj = rbgobj_gobject_new(RVAL2GTYPE(self), params_hash);
+
+    RBGTK_INITIALIZE(self, gobj);
+    return Qnil;
+}
+
+// FIXME: use rb_protect
+static void
+get_prop_func(GObject* object,
+              guint property_id,
+              GValue* value,
+              GParamSpec* pspec)
+{
+    ID ruby_getter = (ID)g_param_spec_get_qdata(pspec, q_ruby_getter);
+    if (!ruby_getter) {
+        gchar* name = g_strdup(g_param_spec_get_name(pspec));
+        gchar* p;
+        for (p = name; *p; p++) {
+          if (*p == '-')
+            *p = '_';
+        }
+        ruby_getter = rb_intern(name);
+        g_param_spec_set_qdata(pspec, q_ruby_getter, (gpointer)ruby_getter);
+        g_free(name);
+    }
+
+    {
+        VALUE ret = rb_funcall(GOBJ2RVAL(object), ruby_getter, 0);
+        rbgobj_rvalue_to_gvalue(ret, value);
+    }
+}
+
+// FIXME: use rb_protect
+static void
+set_prop_func(GObject* object,
+              guint property_id,
+              const GValue* value,
+              GParamSpec* pspec)
+{
+    ID ruby_setter = (ID)g_param_spec_get_qdata(pspec, q_ruby_setter);
+    if (!ruby_setter) {
+        gchar* name = g_strconcat(g_param_spec_get_name(pspec), "=", NULL);
+        gchar* p;
+        for (p = name; *p; p++) {
+          if (*p == '-')
+            *p = '_';
+        }
+        ruby_setter = rb_intern(name);
+        g_param_spec_set_qdata(pspec, q_ruby_setter, (gpointer)ruby_setter);
+        g_free(name);
+    }
+
+    rb_funcall(GOBJ2RVAL(object), ruby_setter, 1, GVAL2RVAL(value));
+}
+
+// FIXME: use rb_protect
+static void
+get_child_prop_func(GtkContainer* container,
+                    GtkWidget*    child,
+                    guint         property_id,
+                    GValue*       value,
+                    GParamSpec*   pspec)
+{
+    ID ruby_getter = (ID)g_param_spec_get_qdata(pspec, q_ruby_getter);
+    if (!ruby_getter) {
+        gchar* name = g_strconcat("get_", g_param_spec_get_name(pspec), NULL);
+        gchar* p;
+        for (p = name; *p; p++) {
+          if (*p == '-')
+            *p = '_';
+        }
+        ruby_getter = rb_intern(name);
+        g_param_spec_set_qdata(pspec, q_ruby_getter, (gpointer)ruby_getter);
+        g_free(name);
+    }
+
+    {
+        VALUE ret = rb_funcall(GOBJ2RVAL(container), ruby_getter, 1, GOBJ2RVAL(child));
+        rbgobj_rvalue_to_gvalue(ret, value);
+    }
+}
+
+// FIXME: use rb_protect
+static void
+set_child_prop_func(GtkContainer* container,
+                    GtkWidget*    child,
+                    guint         property_id,
+                    const GValue* value,
+                    GParamSpec*   pspec)
+{
+    ID ruby_setter = (ID)g_param_spec_get_qdata(pspec, q_ruby_setter);
+    if (!ruby_setter) {
+        gchar* name = g_strconcat("set_", g_param_spec_get_name(pspec), NULL);
+        gchar* p;
+        for (p = name; *p; p++) {
+          if (*p == '-')
+            *p = '_';
+        }
+        ruby_setter = rb_intern(name);
+        g_param_spec_set_qdata(pspec, q_ruby_setter, (gpointer)ruby_setter);
+        g_free(name);
+    }
+
+    rb_funcall(GOBJ2RVAL(container), ruby_setter, 2, GOBJ2RVAL(child), GVAL2RVAL(value));
+}
+
+// FIXME: use rb_protect
+static void
+class_init_func(gpointer g_class, gpointer class_data)
+{
+    GObjectClass* g_class1 = G_OBJECT_CLASS(g_class);
+    GtkContainerClass* g_class2 = GTK_CONTAINER_CLASS(g_class);
+
+    g_class1->set_property = set_prop_func;
+    g_class1->get_property = get_prop_func;
+
+    g_class2->set_child_property = set_child_prop_func;
+    g_class2->get_child_property = get_child_prop_func;
+
+#if 0
+    VALUE class_init_proc = (VALUE)class_data;
+    rb_funcall(proc_mod_eval, rb_intern("call"), 2,
+               GTYPE2CLASS(G_TYPE_FROM_CLASS(g_class)), class_init_proc);
+#endif
+}
+
+static VALUE
+type_register(int argc, VALUE* argv, VALUE self)
+{
+    VALUE type_name, flags;
+    volatile VALUE class_init_proc = Qnil;
+    GType parent_type;
+    GTypeInfo* info;
+
+    rb_scan_args(argc, argv, "03", &type_name, &info, &flags);
+
+    {
+        const RGObjClassInfo* cinfo = rbgobj_lookup_class(self);
+        if (cinfo->klass == self)
+            rb_raise(rb_eTypeError, "already registered");
+    }
+
+    {
+        VALUE superclass = rb_funcall(self, rb_intern("superclass"), 0);
+        const RGObjClassInfo* cinfo = CLASS2CINFO(superclass);
+        if (cinfo->klass != superclass)
+            rb_raise(rb_eTypeError, "super class must be registered to GLib");
+        parent_type = cinfo->gtype;
+    }
+
+    if (NIL_P(type_name)){
+        VALUE s = rb_funcall(self, rb_intern("name"), 0);
+
+        if (strlen(StringValuePtr(s)) == 0)
+            rb_raise(rb_eTypeError, "can't determine type name");        
+
+        type_name = rb_funcall(
+            rb_eval_string("lambda{|x| x.gsub(/::/,'') }"),
+            rb_intern("call"), 1, s);
+    }
+
+    {
+        GTypeQuery query;
+        g_type_query(parent_type, &query);
+
+        info = g_new0(GTypeInfo, 1);
+        info->class_size     = query.class_size;
+        info->base_init      = NULL;
+        info->base_finalize  = NULL;
+        info->class_init     = class_init_func;
+        info->class_finalize = NULL;
+        info->class_data     = (gpointer)class_init_proc;
+        info->instance_size  = query.instance_size;
+        info->n_preallocs    = 0;
+        info->instance_init  = NULL;
+        info->value_table    = NULL;
+    }
+
+    {
+        GType type = g_type_register_static(parent_type,
+                                            StringValuePtr(type_name),
+                                            info,
+                                            NIL_P(flags) ? 0 : NUM2INT(flags));
+        G_RELATIVE(self, class_init_proc);
+
+        rbgobj_register_class(self, type, TRUE, TRUE);
+
+        {
+            RGObjClassInfo* cinfo = (RGObjClassInfo*)rbgobj_lookup_class(self);
+            cinfo->flags |= RBGOBJ_DEFINED_BY_RUBY;
+        }
+
+        {
+            GType parent = g_type_parent(type);
+            const RGObjClassInfo* cinfo = GTYPE2CINFO(parent);
+            VALUE m = rb_define_module_under(self, RubyGtkContainerHookModule);
+
+            if (! (cinfo->flags & RBGOBJ_DEFINED_BY_RUBY)) {
+                rb_define_method(m, "initialize", cont_initialize, -1);
+            }
+
+            rb_include_module(self, m);
+        }
+
+        return Qnil;
+    }
+}
+/**********************************************************************/
+
+
 void 
 Init_gtk_container()
 {
@@ -459,9 +728,11 @@ Init_gtk_container()
 
     rb_define_method(gContainer, "resize_container?", cont_is_resize_container, 0);
     rb_define_method(gContainer, "add", cont_add, -1);
+    rb_define_alias(gContainer, "<<", "add");
     rb_define_method(gContainer, "remove", cont_remove, 1);
     rb_define_method(gContainer, "check_resize", cont_check_resize, 0);
     rb_define_method(gContainer, "each", cont_foreach, -1);
+    rb_define_method(gContainer, "each_forall", cont_forall, -1);
     rb_define_method(gContainer, "children", cont_get_children, 0);
     rb_define_method(gContainer, "set_reallocate_redraws", cont_set_reallocate_redraws, 1);
     rb_define_method(gContainer, "set_focus_child", cont_set_focus_child, 1);
@@ -479,9 +750,16 @@ Init_gtk_container()
     rb_define_method(gContainer, "unset_focus_chain", cont_unset_focus_chain, 0);
     rb_define_singleton_method(gContainer, "child_property", cont_s_child_property, 1);
     rb_define_singleton_method(gContainer, "child_properties", cont_s_child_properties, -1);
-/*
     rb_define_singleton_method(gContainer, "install_child_property", cont_s_install_child_property, -1);
-*/
+
+    q_ruby_getter = g_quark_from_static_string("__ruby_getter");
+    q_ruby_setter = g_quark_from_static_string("__ruby_setter");
+
+    rb_define_singleton_method(gContainer, "type_register", type_register, -1);
+
+    rb_global_variable(&proc_mod_eval);
+    proc_mod_eval = rb_eval_string("lambda{|obj,proc| obj.module_eval(&proc)}");
+
     G_DEF_SETTERS(gContainer);
 
     rb_global_variable(&type_to_prop_setter_table);
