@@ -3,10 +3,10 @@
 
   rbgtkwindow.c -
 
-  $Author: mutoh $
-  $Date: 2003/08/31 15:29:44 $
+  $Author: ggc $
+  $Date: 2005/08/28 20:03:57 $
 
-  Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -22,16 +22,25 @@ gwin_initialize(argc, argv, self)
     VALUE *argv;
     VALUE self;
 {
-    VALUE type;
-    GtkWindowType tp;
+    VALUE arg;
+    GtkWindowType tp = GTK_WINDOW_TOPLEVEL;
+    gchar* title = NULL;
+    GtkWidget* window = NULL;
 
-    if (rb_scan_args(argc, argv, "01", &type) == 1) {
-        tp = RVAL2GENUM(type, GTK_TYPE_WINDOW_TYPE);
-    } else {
-        tp = GTK_WINDOW_TOPLEVEL;
+    if (rb_scan_args(argc, argv, "01", &arg) == 1) {
+        if (TYPE(arg) == T_STRING) {
+            StringValue(arg);
+            title = RVAL2CSTR(arg);
+        } else {
+            tp = RVAL2GENUM(arg, GTK_TYPE_WINDOW_TYPE);
+        }
     }
+    window = gtk_window_new(tp);
 
-    RBGTK_INITIALIZE(self, gtk_window_new(tp));
+    if (title)
+        gtk_window_set_title(GTK_WINDOW(window), title);
+
+    RBGTK_INITIALIZE(self, window);
     return Qnil;
 }
 
@@ -122,6 +131,22 @@ gwin_set_transient_for(self, parent)
     return self;
 }
 
+static VALUE
+gwin_set_destroy_with_parent(self, setting)
+    VALUE self, setting;
+{
+    gtk_window_set_destroy_with_parent(_SELF(self), RTEST(setting));
+    return self;
+}
+
+/* Define as Properties.
+void        gtk_window_set_screen           (GtkWindow *window,
+                                             GdkScreen *screen);
+GdkScreen*  gtk_window_get_screen           (GtkWindow *window);
+gboolean    gtk_window_is_active            (GtkWindow *window);
+gboolean    gtk_window_has_toplevel_focus   (GtkWindow *window);
+*/
+
 static VALUE
 gwin_s_list_toplevels(self)
     VALUE self;
@@ -179,6 +204,22 @@ gwin_set_default(self, win)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,8,0)
+static VALUE
+gwin_present(argc, argv, self)
+    int argc;
+    VALUE *argv;
+    VALUE self;
+{
+    VALUE timestamp;
+    if (rb_scan_args(argc, argv, "01", &timestamp) == 1) {
+        gtk_window_present_with_time(_SELF(self), NUM2UINT(timestamp));
+    } else {
+        gtk_window_present(_SELF(self));
+    }
+    return self;
+}
+#else
 static VALUE
 gwin_present(self)
     VALUE self;
@@ -186,6 +227,7 @@ gwin_present(self)
     gtk_window_present(_SELF(self));
     return self;
 }
+#endif
 
 static VALUE
 gwin_iconify(self)
@@ -235,6 +277,39 @@ gwin_unmaximize(self)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,2,0) 
+static VALUE
+gwin_fullscreen(self)
+    VALUE self;
+{
+    gtk_window_fullscreen(_SELF(self));
+    return self;
+}
+static VALUE
+gwin_unfullscreen(self)
+    VALUE self;
+{
+    gtk_window_unfullscreen(_SELF(self));
+    return self;
+}
+#endif
+#if GTK_CHECK_VERSION(2,4,0)
+static VALUE
+gwin_set_keep_above(self, setting)
+    VALUE self, setting;
+{
+    gtk_window_set_keep_above(_SELF(self), RTEST(setting));
+    return self;
+}
+static VALUE
+gwin_set_keep_below(self, setting)
+    VALUE self, setting;
+{
+    gtk_window_set_keep_below(_SELF(self), RTEST(setting));
+    return self;
+}
+#endif
+
 static VALUE
 gwin_begin_resize_drag(self, edge, button, root_x, root_y, timestamp)
     VALUE self, edge, button, root_x, root_y, timestamp;
@@ -305,6 +380,27 @@ gwin_set_type_hint(self, hint)
     gtk_window_set_type_hint(_SELF(self), RVAL2GENUM(hint, GDK_TYPE_WINDOW_TYPE_HINT));
     return self;
 }
+/* Defined as Properties
+void        gtk_window_set_skip_taskbar_hint
+                                            (GtkWindow *window,
+                                             gboolean setting);
+void        gtk_window_set_skip_pager_hint  (GtkWindow *window,
+                                             gboolean setting);
+void        gtk_window_set_accept_focus     (GtkWindow *window,
+                                             gboolean setting);
+
+gchar*      gtk_window_get_icon_name        (GtkWindow *window);
+void        gtk_window_set_icon_name        (GtkWindow *window,
+                                             const gchar *name);
+
+void        gtk_window_set_focus_on_map     (GtkWindow *window,
+                                             gboolean setting);
+gboolean    gtk_window_get_focus_on_map     (GtkWindow *window);
+
+void        gtk_window_set_urgency_hint     (GtkWindow *window,
+                                             gboolean setting);
+*/
+
 
 static VALUE
 gwin_get_decorated(self)
@@ -457,6 +553,59 @@ gwin_s_set_default_icon_list(self, list)
     return list;
 }
 
+#if GTK_CHECK_VERSION(2,2,0)
+static VALUE
+gwin_s_set_default_icon(self, icon_or_filename)
+    VALUE self, icon_or_filename;
+{
+    if (TYPE(icon_or_filename) == T_STRING){
+        GError* err;
+        gboolean ret = gtk_window_set_default_icon_from_file(RVAL2CSTR(icon_or_filename), &err);
+        if (! ret)
+            RAISE_GERROR(err);
+    } else {
+#if GTK_CHECK_VERSION(2,4,0)
+        gtk_window_set_default_icon(GDK_PIXBUF(RVAL2GOBJ(icon_or_filename)));
+#else
+        rb_raise(rb_eArgError, "Invalid argument: %s, or you may need to use GTK+-2.4.x", 
+                 rb_class2name(CLASS_OF(icon_or_filename)));
+#endif
+    }
+    return self;
+}
+#endif
+
+#if GTK_CHECK_VERSION(2,6,0)
+static VALUE
+gwin_s_set_default_icon_name(self, name)
+    VALUE self, name;
+{
+    gtk_window_set_default_icon_name(RVAL2CSTR(name));
+    return self;
+}
+#endif
+
+static VALUE
+gwin_set_icon(self, icon_or_filename)
+    VALUE self, icon_or_filename;
+{
+    if (TYPE(icon_or_filename) == T_STRING){
+#if GTK_CHECK_VERSION(2,2,0)
+        GError* err;
+        gboolean ret = gtk_window_set_icon_from_file(_SELF(self),
+                                                     RVAL2CSTR(icon_or_filename), &err);
+        if (! ret)
+            RAISE_GERROR(err);
+#else
+        rb_raise(rb_eArgError, "Invalid argument: %s, or you may need to use GTK+-2.4.x", 
+                 rb_class2name(CLASS_OF(icon_or_filename)));
+#endif
+    } else {
+        gtk_window_set_icon(_SELF(self), GDK_PIXBUF(RVAL2GOBJ(icon_or_filename)));
+    }
+    return self;
+}
+
 static VALUE
 gwin_set_icon_list(self, list)
     VALUE self, list;
@@ -471,6 +620,15 @@ gwin_set_icon_list(self, list)
     g_list_free(glist);
     return list;
 }
+#if GTK_CHECK_VERSION(2,2,0)
+static VALUE
+gwin_s_set_auto_startup_notification(self, setting)
+    VALUE self, setting;
+{
+    gtk_window_set_auto_startup_notification(RTEST(setting));
+    return self;
+}
+#endif
 
 /* They are not public methods.
 static VALUE
@@ -523,6 +681,7 @@ Init_gtk_window()
     rb_define_method(gWindow, "set_gravity", gwin_set_gravity, 1);
     rb_define_method(gWindow, "gravity", gwin_get_gravity, 0);
     rb_define_method(gWindow, "set_transient_for", gwin_set_transient_for, 1);
+    rb_define_method(gWindow, "set_destroy_with_parent", gwin_set_destroy_with_parent, 1);
     rb_define_singleton_method(gWindow, "toplevels", gwin_s_list_toplevels, 0);
     rb_define_method(gWindow, "add_mnemonic", gwin_add_mnemonic, 2);
     rb_define_method(gWindow, "remove_mnemonic", gwin_remove_mnemonic, 2);
@@ -530,13 +689,25 @@ Init_gtk_window()
     rb_define_method(gWindow, "focus", gwin_get_focus, 0);
     rb_define_method(gWindow, "set_focus", gwin_set_focus, 1);
     rb_define_method(gWindow, "set_default", gwin_set_default, 1);
+#if GTK_CHECK_VERSION(2,8,0)
+    rb_define_method(gWindow, "present", gwin_present, -1);
+#else
     rb_define_method(gWindow, "present", gwin_present, 0);
+#endif
     rb_define_method(gWindow, "iconify", gwin_iconify, 0);
     rb_define_method(gWindow, "deiconify", gwin_deiconify, 0);
     rb_define_method(gWindow, "stick", gwin_stick, 0);
     rb_define_method(gWindow, "unstick", gwin_unstick, 0);
     rb_define_method(gWindow, "maximize", gwin_maximize, 0);
     rb_define_method(gWindow, "unmaximize", gwin_unmaximize, 0);
+#if GTK_CHECK_VERSION(2,2,0)
+    rb_define_method(gWindow, "fullscreen", gwin_fullscreen, 0);
+    rb_define_method(gWindow, "unfullscreen", gwin_unfullscreen, 0);
+#endif
+#if GTK_CHECK_VERSION(2,4,0)
+    rb_define_method(gWindow, "set_keep_above", gwin_set_keep_above, 1);
+    rb_define_method(gWindow, "set_keep_below", gwin_set_keep_below, 1);
+#endif
     rb_define_method(gWindow, "begin_resize_drag", gwin_begin_resize_drag, 5);
     rb_define_method(gWindow, "begin_move_drag", gwin_begin_move_drag, 4);
     rb_define_method(gWindow, "set_decorated", gwin_set_decorated, 1);
@@ -563,7 +734,19 @@ Init_gtk_window()
     rb_define_method(gWindow, "reshow_with_initial_size", gwin_reshow_with_initial_size, 0);
     rb_define_method(gWindow, "resize", gwin_resize, 2);
     rb_define_singleton_method(gWindow, "set_default_icon_list", gwin_s_set_default_icon_list, 1);
+#if GTK_CHECK_VERSION(2,2,0)
+    rb_define_singleton_method(gWindow, "set_default_icon", gwin_s_set_default_icon, 1);
+#endif
+#if GTK_CHECK_VERSION(2,6,0)
+    rb_define_singleton_method(gWindow, "set_default_icon_name", gwin_s_set_default_icon_name, 1);
+#endif
+    rb_define_method(gWindow, "set_icon", gwin_set_icon, 1);
     rb_define_method(gWindow, "set_icon_list", gwin_set_icon_list, 1);
+#if GTK_CHECK_VERSION(2,2,0)
+    rb_define_singleton_method(gWindow, "set_auto_startup_notification", 
+                               gwin_s_set_auto_startup_notification, 1);
+#endif
+
 /*
     rb_define_method(gWindow, "decorated_window_init", gwin_decorated_window_init, 0);
     rb_define_method(gWindow, "decorated_window_calculate_frame_size", gwin_decorated_window_calculate_frame_size, 0);   
